/*
ID: espr1t
TASK: SuperMario
KEYWORDS: Testgen
*/

#include <cstdio>
#include <queue>
#include <map>
#include <random>
#include <ctime>
#include <cstring>

using namespace std;

const int MAX_N = 10000002;
const int MAX_K = 1000002;
const long long INF = 1000000000000000001LL;

int n, k;
int a[MAX_N];
mt19937 random;

int findCycle() {
    map <int, int> seen;
    for (int i = 0; i < n && i < 10000000; i++) {
        if (seen.find(a[i]) != seen.end()) {
            fprintf(stderr, "  -- cycle length %d (with n = %d, k = %d)\n",
                i - seen[a[i]], n, k);
            return i - seen[a[i]];
        }
        seen[a[i]] = i;
    }
    fprintf(stderr, "  -- no cycle (n = %d, k = %d)\n", n, k);
    return n;
}

void generate(int F, int A, int B, int M) {
    a[0] = F;
    for (int i = 1; i < n; i++)
        a[i] = (int)(((long long)a[i - 1] * A + B) % M);
    findCycle();
}

long long solve4(int n, int k, int F, int A, int B, int M) {
    long long ans = (long long)n * M;
    deque < pair <long long, int> > q(1, make_pair(0, -1));
    for (int i = 0; i < n; i++) {
        while (!q.empty() && i - k > q.front().second)
            q.pop_front();
        pair <long long, int> cur = make_pair(F + q.front().first, i);
        F = (int)(((long long)F * A + B) % M);
        while (!q.empty() && q.back().first >= cur.first)
            q.pop_back();
        q.push_back(cur);
        if (i + k >= n)
            ans = min(ans, cur.first);
    }
    return k > n ? 0 : ans;
}

const int TREE = 2097152;
long long tree[TREE];

void update(int idx, long long val) {
    idx += (TREE >> 1);
    tree[idx] = val; idx >>= 1;
    while (idx) {
        long long nval = min(tree[(idx << 1) | 0], tree[(idx << 1) | 1]);
        if (tree[idx] == nval)
            break;
        tree[idx] = nval, idx >>= 1;
    }
}

long long solve3(int n, int k, int F, int A, int B, int M) {
    long long ans = INF;
    for (int i = 0; i < TREE; i++)
        tree[i] = INF;
    update(k - 1, 0);
    for (int i = 0; i < n; i++) {
        long long cur = F + tree[1];
        F = (int)(((long long)F * A + B) % M);
        update(i % k, cur);
        if (i + k >= n)
            ans = min(ans, cur);
    }
    return k > n ? 0 : ans;
}

long long solve2(int n, int k, int F, int A, int B, int M) {
    long long ans = INF;
    priority_queue < pair <long long, int> > q;
    q.push(make_pair(0, -1));
    for (int i = 0; i < n; i++) {
        /*
        if ((int)q.size() > 4000000) {
            vector < pair <long long, int> > v;
            v.reserve(k);
            while (!q.empty()) {
                if (q.top().second >= i - k)
                    v.push_back(q.top());
                q.pop();
            }
            q = priority_queue< pair <long long, int> >(v.begin(), v.end());
        }
        */
        
        while (!q.empty() && q.top().second < i - k)
            q.pop();
        long long cur = F - q.top().first;
        F = (int)(((long long)F * A + B) % M);
        if (i + k >= n)
            ans = min(ans, cur);
        q.push(make_pair(-cur, i));
    }
    return k > n ? 0 : ans;
}

long long solve1() {
    long long ans = INF;
    static long long dyn[MAX_K];
    memset(dyn, 0, sizeof(dyn));
    
    for (int i = 0; i < n; i++) {
        long long& cur = dyn[i % k];
        cur += a[i];
        for (int c = k; c > 0; c--)
            cur = min(cur, a[i] + dyn[(i - c + k) % k]);
        if (i + k >= n)
            ans = min(ans, cur);
    }
    return k > n ? 0 : ans;
}

int main(void) {
    unsigned sTime;
    /*
    int testSize[21] = {
        20, // Example
        42, 100, 500, 1000, 2000, 5000, // Group 1
        10000, 50000, 200000, 1000000, 5000000, 10000000, // Group 2
        20000000, 25000000, 30000000, 35000000, 40000000, 45000000, 50000000, 50000000 // Group 3
    };
    */
    /*
    int testSize[21] = {
        20, // Example
        42, 100, 500, 1000, 2000, 5000, // Group 1
        10000, 50000, 200000, 100000, 500000, 1000000, // Group 2
        2000000, 5000000, 6000000, 7000000, 8000000, 9000000, 10000000, 10000000 // Group 3
    };
    */
    int testSize[21] = {
        20, // Example
        42, 100, 200, 300, 500, 1000, 2000, 3000, 4000, 5000, // Group 1
        30000, 50000, 100000, 200000, 500000, 1000000, // Group 2
        5000000, 8000000, 10000000, 10000000 // Group 3
    };
    
    for (int t = 0; t <= 20; t++) {
        n = testSize[t], k = ((random() % 1000000) % n) + 1;
        int M = random() % 1000000007 + 1;
        if (random() % 2 == 0)
            M = 1000000007;
        int F = random() % M;
        int A = random() % M;
        int B = random() % M;

        if (t == 0) {
            n = 20, k = 5;
            F = 7; A = 3; B = 8; M = 23;
        } else if (t == 2) {
            k = n + 3;
        } else if (t == 10 || t == 17) {
            A = 1, B = 2;
        } else if (t == 11 || t == 18) {
            A = 1, B = 0;
        } else if (t == 13) {
            k = 42;
        } else if (t == 15) {
            A = 1, B = 13;
        } else if (t == 20) {
            k = 1337;
        }

        fprintf(stderr, "Testcase %d with n = %d, k = %d, mod = %d\n", t, n, k, M);
        generate(F, A, B, M);

        
        if ((long long)n * k <= 200000000LL) {
            sTime = clock();
            long long ans1 = solve1();
            fprintf(stderr, "  >> dummy answer = %lld (execution time: %.3lfs)\n",
                ans1, (double)(clock() - sTime) / CLOCKS_PER_SEC);
        }
        
        sTime = clock();
        long long ans2 = solve2(n, k, F, A, B, M);
        fprintf(stderr, "  >> pqueue answer = %lld (execution time: %.3lfs)\n",
            ans2, (double)(clock() - sTime) / CLOCKS_PER_SEC);
        
        sTime = clock();
        long long ans3 = solve3(n, k, F, A, B, M);
        fprintf(stderr, "  >> itree answer = %lld (execution time: %.3lfs)\n",
            ans3, (double)(clock() - sTime) / CLOCKS_PER_SEC);

        sTime = clock();
        long long ans4 = solve4(n, k, F, A, B, M);
        fprintf(stderr, "  >> smarty answer = %lld (execution time: %.3lfs)\n",
            ans4, (double)(clock() - sTime) / CLOCKS_PER_SEC);
        
        if (ans4 != ans3)
            fprintf(stderr, "!!!!!!!ERROR: ans4 != ans 3 (%lld vs %lld)\n", ans4, ans3);

        char inpFile[32], solFile[32];
        sprintf(inpFile, "SuperMario.%02d.in", t);
        sprintf(solFile, "SuperMario.%02d.sol", t);

        FILE* in = fopen(inpFile, "wt");
        fprintf(in, "%d %d\n", n, k);
        fprintf(in, "%d %d %d %d\n", F, A, B, M);
        fclose(in);
        
        FILE* out = fopen(solFile, "wt");
        fprintf(out, "%lld\n", ans4);
        fclose(out);
    }
    return 0;
}
